library IEEE;
use IEEE.STD_LOGIC_1164.ALL;



entity FactorielN is
	Generic ( N : positive := 108; 
			  M : positive := 5
			);
    Port ( Rst 			: in  STD_LOGIC;
           Clk 			: in  STD_LOGIC;
           D_in 		: in  STD_LOGIC_VECTOR (M-1 downto 0);
		   D_out_Ready 	: out  STD_LOGIC;
           D_out 		: out  STD_LOGIC_VECTOR (N+M-1 downto 0));
end FactorielN;

architecture Behavioral of FactorielN is

signal D_out_CMP_tmp 	:   STD_LOGIC :='0';
signal D_out_CMP 		:   STD_LOGIC :='0';
signal D_outReady 		:   STD_LOGIC :='0';
signal Rst_tmp 			:   STD_LOGIC :='0';
signal D_out_count 		:   STD_LOGIC_VECTOR (M-1 downto 0):=(others =>'0');
signal D_out_tmp 		:   STD_LOGIC_VECTOR (N+M-1 downto 0):=x"000000000000000000000000000" & b"00001";
signal Data_in_A_tmp 	:   STD_LOGIC_VECTOR (M-1 downto 0):=b"00001";



-- CMP 
COMPONENT CountN
PORT(
	Data_in : IN std_logic_vector(M-1 downto 0);
	Rst : IN std_logic;
	Clk : IN std_logic;          
	End_count : OUT std_logic;
	Count_out : OUT std_logic_vector(M-1 downto 0)
	);
END COMPONENT;
	
COMPONENT MulAccN
PORT(
	Data_in_A : IN std_logic_vector(M-1 downto 0);
	Rst : IN std_logic;
	Clk : IN std_logic;       
	Data_out_P : INOUT std_logic_vector(N+M-1 downto 0)
	);
END COMPONENT;
	

begin
	
	-- Counter
	Count1: CountN PORT MAP(
		Data_in => D_in,
		Rst => Rst,
		Clk => Clk,
		End_count => D_outReady,
		Count_out => D_out_count 
	);
	
	-- Mul-ACC
	MULACC_1: MulAccN PORT MAP(
		Data_in_A => Data_in_A_tmp,
		Rst => Rst_tmp,
		Clk => Clk,
		Data_out_P => D_out_tmp
	);
	Data_in_A_tmp <= D_out_count;
	Rst_tmp <= D_outReady OR Rst; 
	D_out_Ready <= D_outReady;
	
	
	-- Latence d'une periode 
	P_data_out : process(Rst, Clk)
	begin
		if Rst ='1' then 
			D_out(N+M-1 downto 1) <=(others =>'0');
			D_out(0) <= '1'; 
		elsif Clk = '1' and Clk'event then 
			D_out <= D_out_tmp;
		end if;
	end process;
	
end Behavioral;

